searchData={"items":[{"type":"module","title":"Hop","doc":"Hop is a tiny web crawling framework for Elixir.\n\nHop's goal is to be simple and extensible, while still providing\nenough guardrails to get up and running quickly. Hop implements a simple,\ndepth-limited, breadth-first crawler - and keeps track of already visited\nURLs for you. It also provides several utility functions for implementing\ncrawlers with best practices.\n\nYou can crawl a webpage with just a few lines of code with Hop:\n\n    url\n    |> Hop.new()\n    |> Hop.stream()\n    |> Enum.each(fn {url, _response, _state} ->\n      IO.puts(\"Visited: #{url}\")\n    end)\n\n\nBy default, Hop will perform a single-host, breadth-first crawl of all of the\npages on the website. The `stream/2` execution function will return a stream of\n`{url, response, state}` tuples for each successfully visited page in the crawl.\nThis is designed to make it easy to perform caching, extract items from a page,\nor do whatever other work deemed necessary.\n\nThe defaults are designed to get you up and running quickly; however, the power comes\nin with Hop's simple extensibility. Hop provides extensibility by allowing you to\ncustomize your crawler's behavior at 3 different steps:\n\n  1. Prefetch\n  2. Fetch\n  3. Next","ref":"Hop.html"},{"type":"module","title":"Prefetch - Hop","doc":"The prefetch stage is a pre-request stage typically used for request validation.\nBy default, during the prefetch stage, Hop will:\n\n  1. Check that the URL has a populated scheme and host\n\n  2. Check that the URL's scheme is acceptable. Default accepted schemes\n  are http and https. This prevents Hop from attempting to visit `tel:`,\n  `email:` and other links with non-HTTP schemes.\n\n  3. Check that the host is actually valid, using `:inet.gethostbyname`\n\n  4. Check that the host is an acceptable host to visit, e.g. not outside of\n  the host the crawl started on.\n\n  5. Perform a HEAD request to check that the content-length does not exceed\n  the maximum specified content length and to check that the content-type matches\n  one of the acceptable mime-types. This is useful for preventing Hop from spending\n  time downloading large files you don't care about.\n\nMost users should stick to using Hop's default pre-request logic. If you want to\ncustomize the behavior; however, you can pass your own `prefetch/3` function:\n\n    url\n    |> Hop.new()\n    |> Hop.prefetch(fn url, _state, _opts -> {:ok, url} end)\n    |> Hop.stream()\n    |> Enum.each(fn {url, _response, _state} ->\n      IO.puts(\"Visited: #{url}\")\n    end)\n\nThis simple example performs no validation, and forwards all URLs to the fetch stage.\n\nYour custom `prefetch/3` function should be an arity-3 function which takes a URL, the\ncurrent Crawl state, and the current Hop's configuration options as input and returns\n`{:ok, url}` or an error value. Any none `{:ok, url}` values will be ignored during fetch.","ref":"Hop.html#module-prefetch"},{"type":"module","title":"Fetch - Hop","doc":"The fetch stage performs the actual requests during your crawl. By default, Hop\nuses Req and performs a GET request without retries on each URL. In other words,\nthe default fetch function in Hop is:\n\n    def fetch(url, state, opts) do\n      req_options = opts[:req_options]\n\n      with {:ok, response} <- Req.get(url, req_options) do\n        {:ok, response, state}\n      end\n    end\n\nNotice you can customize the `Req` request options by passing `:req_options` as\na part of your Hop's configuration.\n\nThis is simple and acceptable for many cases; however, certain applications require\nmore advanced setups and customization options. For example, you might want to configure\nyour fetch to proxy requests through Puppeteer to render javascript. You can do this easily:\n\n    def custom_fetch(url, state, _opts) do\n      # url to proxy that renders a page with puppeteer\n      proxy_url = \"http://localhost:3000/render\"\n\n      with {:ok, response} <- Req.post(url, json: %{url: url}) do\n        {:ok, response, state}\n      end\n    end","ref":"Hop.html#module-fetch"},{"type":"module","title":"Next - Hop","doc":"The next function dictates the next links to be crawled during execution.\nIt takes as input the current URL, the response from fetch, the current state,\nand configuration options. By default, Hop returns all links on the current page\nas next in the queue to be crawled. You can customize this behaviour by implementing\nyour own `next/4` function:\n\n    def custom_next(url, response, state, _opts) do\n      links =\n        url\n        |> Hop.fetch_links(response)\n        |> Enum.reject(&String.contains?(&1, \"wp-uploads\"))\n\n      {:ok, links, state}\n    end\n\nThis simple example ignores all URLs that contain `wp-uploads`. Hop provides a convenience\n`fetch_links/2` to fetch all of the absolute URLs on a webpage. This just uses Floki under-the-hood.","ref":"Hop.html#module-next"},{"type":"function","title":"Hop.config/2","doc":"Returns the current configuration value set for the given\nkey in the Hop.","ref":"Hop.html#config/2"},{"type":"function","title":"Hop.fetch/2","doc":"Sets this hop's fetch function.\n\nThe fetch function is what is used to actually make requests. By\ndefault, Hop uses `&Req.get(&1, retry: false)`. If you want to\nchange the options passed to Req, you can do so here.\n\nYour `fetch/1` function should accept a URL and return a tuple of\n`{:ok, response}`.\n\nNote that Hop is HTTP-Client agnostic. The `response` object is\nsimply forwarded to the `process` function. This means you can\nswap for a new HTTP-client if necessary.","ref":"Hop.html#fetch/2"},{"type":"function","title":"Hop.fetch_links/3","doc":"Fetches all of the links on a given page.\n\nThis function takes the current URL and merges it with the given\nanchor link to generate a fully-qualified URL.","ref":"Hop.html#fetch_links/3"},{"type":"function","title":"Options - Hop.fetch_links/3","doc":"* `:crawl_query?` - whether or not to treat query parameters\n    as unique links to crawl. Defaults to `true`\n\n    * `:crawl_fragment?` - whether or not to treat fragments as\n    unique links to crawl. Defaults to `false`","ref":"Hop.html#fetch_links/3-options"},{"type":"function","title":"Hop.new/2","doc":"Creates a new Hop starting at the given URL(s).","ref":"Hop.html#new/2"},{"type":"function","title":"Hop.next/2","doc":"Sets this hop's next function.\n\nThe next function dictates which links are meant to be crawled\nnext after the current page.","ref":"Hop.html#next/2"},{"type":"function","title":"Hop.prefetch/2","doc":"Sets this hop's prefetch function.\n\nThe prefetch function is essentially meant to be a pre-request\nvalidation stage. This could serve the purpose of validating that\na given URL is valid, that the content is valid (e.g. via a HEAD\nrequest), that the request matches a site's Robots.txt, etc.\nMost clients will want to leave this as-is.","ref":"Hop.html#prefetch/2"},{"type":"function","title":"Hop.put_config/3","doc":"Puts the given configuration value for the given key\nin the given hop.","ref":"Hop.html#put_config/3"},{"type":"function","title":"Hop.stream/2","doc":"Returns a stream that represents the execution of the given Hop.\n\nThis function will perform a limited-depth, breadth-first crawl from\nthe given start URL, and lazily return tuples of `{url, response, state}`\nfor each successfully visited page.","ref":"Hop.html#stream/2"},{"type":"function","title":"Hop.validate_content/3","doc":"Validates that the given URL's content is valid for the crawl.\n\nThis function attempts to perform a HEAD request to the given URL to\ncheck if the response content-type is one of the accepted mime types\nand that the max content length is below the specified max content length\nfor the crawl.\n\nIf the given server does not support HEAD requests, it will simply\naccept the URL as valid.","ref":"Hop.html#validate_content/3"},{"type":"function","title":"Hop.validate_hostname/3","doc":"Validates that the given URL's hostname is valid for the crawl.\n\nValidates that the hostname is populated, correct, and falls within\nthe set of hostnames allowed according to the crawl state.","ref":"Hop.html#validate_hostname/3"},{"type":"function","title":"Hop.validate_scheme/3","doc":"Validates that the given URL's scheme is valid for the crawl.\n\nValidates that the scheme is populated, correct, and falls within\none of the configured accepted schemes according to the `:accepted_schemes`\nconfiguration option.","ref":"Hop.html#validate_scheme/3"},{"type":"function","title":"Hop.validate_visited/3","doc":"Validates that the given URL has not already been visited.\n\nThe Crawl state contains a member `:visited` that is populated with\na set of URLs that have already been visited.","ref":"Hop.html#validate_visited/3"},{"type":"function","title":"Hop.visit/2","doc":"Marks the given URL as visited.\n\nThis will update the set of visited URLs, and also mark this URL\nas the last crawled URL in the state struct.","ref":"Hop.html#visit/2"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}